// transfer ARMO CLOT from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
  bmdt, formid: Cardinal;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // ClothesFarmClothes04 "Clothes" [ARMO:0003452E]
  
  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    
    formid := $00012E4D; // default - iron helmet
    bmdt := StrToInt(slv[4]);
    // left or right ring, JewelryRingSilver "Silver Ring" [ARMO:0003B97C]
    if (bmdt and ($40 + $80)) > 0 then formid := $0003B97C else
    // amulet, JewelryNecklaceSilver "Silver Necklace" [ARMO:0009171B]
    if (bmdt and $100) > 0 then formid := $0009171B;
    
    // check Armor Value for armor or clothing
    if slv[6] <> '' then begin
      // head or hair, ArmorIronHelmet "Iron Helmet" [ARMO:00012E4D]
      if (bmdt and ($1 + $2)) > 0 then formid := $00012E4D else
      // upper or lower body, ArmorIronCuirass "Iron Armor" [ARMO:00012E49]
      if (bmdt and ($4 + $8)) > 0 then formid := $00012E49 else
      // hands, ArmorIronGauntlets "Iron Gauntlets" [ARMO:00012E46]
      if (bmdt and $10) > 0 then formid := $00012E46 else
      // foot, ArmorIronBoots "Iron Boots" [ARMO:00012E4B]
      if (bmdt and $20) > 0 then formid := $00012E4B else
      // shield, ArmorIronShield "Iron Shield" [ARMO:00012EB6]
      if (bmdt and $2000) > 0 then formid := $00012EB6;
    end else begin
      // head or hair, ClothesFarmHat01 "Hat" [ARMO:000330B3]
      if (bmdt and ($1 + $2)) > 0 then formid := $000330B3 else
      // upper or lower body, ClothesFarmClothes01 "Belted Tunic" [ARMO:0001BE1A]
      if (bmdt and ($4 + $8)) > 0 then formid := $0001BE1A else
      // hands, ClothesPsiijicGloves "Psiijic Gloves" [ARMO:00065B9D]
      if (bmdt and $10) > 0 then formid := $00065B9D else
      // foot, ClothesFarmBoots01 "Boots" [ARMO:0001BE1B]
      if (bmdt and $20) > 0 then formid := $0001BE1B;
    end;
    
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Result := 1;
      Exit;
    end;
    
    
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    if slv[2] <> '' then
      SetElementEditValues(r, 'FULL', slv[2]);
    if slv[3] <> '' then
      SetElementNativeValues(r, 'EITM', StrToInt64('$' + slv[3]));
    if slv[6] <> '' then
      SetElementEditValues(r, 'DNAM', slv[6]);
    SetElementEditValues(r, 'DATA\Value', slv[7]);
    SetElementEditValues(r, 'DATA\Weight', slv[8]);
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind ARMO CLOT.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  enam: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if (Signature(e) <> 'ARMO') and (Signature(e) <> 'CLOT') then
    Exit;

  enam := IntToHex(GetElementNativeValues(e, 'ENAM'), 8);
  if Copy(enam, 1, 2) <> '01' then enam := '';
  line := Format('%s;%s;%s;%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    enam,
    IntToStr(GetElementNativeValues(e, 'BMDT\Biped Flags')),
    IntToStr(GetElementNativeValues(e, 'BMDT\General Flags')),
    GetElementEditValues(e, 'DATA\Armor'),
    GetElementEditValues(e, 'DATA\Value'),
    GetElementEditValues(e, 'DATA\Weight')
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
