// transfer BOOK from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

  
function OneLineStr(s: string): string;
begin
  Result := s;
  Result := StringReplace(Result, #13, '#13', [rfReplaceAll]);
  Result := StringReplace(Result, #10, '#10', [rfReplaceAll]);
  Result := StringReplace(Result, ';', '#59', [rfReplaceAll]);
end;

function MultiLineStr(s: string): string;
begin
  Result := s;
  Result := StringReplace(Result, '#13', #13, [rfReplaceAll]);
  Result := StringReplace(Result, '#10', #10, [rfReplaceAll]);
  Result := StringReplace(Result, '#59', ';', [rfReplaceAll]);
end;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  base_book, base_note: IInterface;
  i: integer;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // MS01WeylinNote "Weylin's Note" [BOOK:000D670F]
  base_note := RecordByFormID(FileByIndex(0), $000D670F, True);
  if not Assigned(base_note) then begin
    AddMessage('Can not find base record note');
    Result := 1;
    Exit;
  end;

  // Book2CommonBeggarPrince "Beggar Prince" [BOOK:0001ACC7]
  base_book := RecordByFormID(FileByIndex(0), $0001ACC7, True);
  if not Assigned(base_book) then begin
    AddMessage('Can not find base record book');
    Result := 1;
    Exit;
  end;

  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];

    // has scroll flag from oblivion? then should be note record
    if (StrToInt(slv[4]) and 1) > 0 then
      baserecord := base_note
    else
      baserecord := base_book;
    
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    SetElementEditValues(r, 'FULL', slv[2]);
    if slv[3] <> '' then
      SetElementEditValues(r, 'Model\MODL', slv[3]);
    SetElementEditValues(r, 'DATA\Weight', slv[5]);
    SetElementEditValues(r, 'DATA\Value', slv[6]);
    SetElementEditValues(r, 'DESC', MultiLineStr(slv[7]));
    //Exit;
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind BOOK.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'BOOK' then
    Exit;

  // export
  line := Format('%s;%s;%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    IntToStr(GetElementNativeValues(e, 'DATA\Flags')),
    GetElementEditValues(e, 'DATA\Weight'),
    GetElementEditValues(e, 'DATA\Value'),
    OneLineStr(GetElementEditValues(e, 'DESC'))
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
