// prepare REGN CELL WRLD groups to be moved to TES5Edit
// remove binary incompatible subrecords, dump valuable info

unit UserScript;

const
  plugin_ver = 0.94;

var
  fname: string;
  sl, slv: TStringList;
  slCELL, slREFR: TStringList;
  slObl, slSky: TStringList;

//============================================================================
function Obl2Sky(FormID: Cardinal): Cardinal;
var
  i: integer;
begin
  if not Assigned(slObl) then begin
    //AddMessage('Initializing converter');
    slObl := TStringList.Create;
    slSky := TStringList.Create;
    sl.LoadFromFile(ProgramPath + 'Edit Scripts\OblivionSkyrim.txt');
    for i := 0 to sl.Count - 1 do begin
      slv.DelimitedText := sl[i];
      if slv.Count > 2 then begin
        slObl.Add(slv[0]);
        slSky.Add(slv[2]);
      end;
    end;
  end;
  
  if (FormID shr 24) > 0 then begin
    Result := FormID;
    Exit;
  end;
  
  i := slObl.IndexOf(IntToHex64(FormID, 8));
  if i <> -1 then
    Result := StrToInt64('$' + slSky[i])
  else
    //GlazedCup01 "Cup" [MISC:000B9BDC]
    Result := $000B9BDC;

end;


//============================================================================
procedure BeforeTES4(e: IInterface);
var
  i: integer;
  masters, m: IInterface;
begin
  SetElementNativeValues(e, 'HEDR\Version', plugin_ver);
  // rename oblivion.esm to skyrim.esm, remove anything else
  masters := ElementByName(e, 'Master Files');
  for i := ElementCount(masters) - 1 downto 0 do begin
    m := ElementByIndex(masters, i);
    if SameText(GetElementEditValues(m, 'MAST'), 'Oblivion.esm') then
      SetElementEditValues(m, 'MAST', 'Skyrim.esm')
    else
      RemoveElement(masters, i);
  end;
end;  

//============================================================================
procedure BeforeREGN(e: IInterface);
var
  i: integer;
  rdata, ent: IInterface;
begin
  // remove everything from Region Data Entries except Map
  rdata := ElementByName(e, 'Region Data Entries');
  for i := ElementCount(rdata) - 1 downto 0 do begin
    ent := ElementByIndex(rdata, i);
    if GetElementEditValues(ent, 'RDAT\Type') <> 'Map' then
      RemoveElement(rdata, i);
  end;
end;  

//============================================================================
procedure BeforeWRLD(e: IInterface);
var
  flags: Word;
begin
  // remove climate
  RemoveElement(e, 'CNAM');
  // remove water
  RemoveElement(e, 'NAM2');
  // remove music
  RemoveElement(e, 'SNAM');
  // fix flags
  flags := GetElementNativeValues(e, 'DATA');
  // clear oblivion worldspace flag
  flags := flags and $FFFB;
  // move 'No Lod Water' flag from bit $10 to $08
  if (flags and $10) > 0 then begin
    flags := flags and $FFEF;
    flags := flags or $08;
  end;
  SetElementNativeValues(e, 'DATA', flags);
end;  

//============================================================================
procedure BeforeCELL(e: IInterface);
var
  line: string;
  flags: Word;
begin
  flags := GetElementNativeValues(e, 'DATA');
  // remove 'Oblivion interior' flag from interior cells
  if (flags and $01) > 0 then
    flags := flags and $FFF7;

  line := Format('%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    IntToHex(flags, 4), // reassign flags in TES5Edit to be 2 bytes long
    GetElementEditValues(e, 'XCMT'), // save music type to later determine type of cell (dungeon, public)
    GetElementEditValues(e, 'XCLW')  // XCLW is in wrong order, will be recreated
  ]);
  slCELL.Add(line);

  if ElementExists(e, 'XCLL') then begin
    slCELL.Add('XCLL');
    slCELL.Add('XCLL\Ambient Color\Red;' + GetElementEditValues(e, 'XCLL\Ambient Color\Red'));
    slCELL.Add('XCLL\Ambient Color\Green;' + GetElementEditValues(e, 'XCLL\Ambient Color\Green'));
    slCELL.Add('XCLL\Ambient Color\Blue;' + GetElementEditValues(e, 'XCLL\Ambient Color\Blue'));
    slCELL.Add('XCLL\Directional Color\Red;' + GetElementEditValues(e, 'XCLL\Directional Color\Red'));
    slCELL.Add('XCLL\Directional Color\Green;' + GetElementEditValues(e, 'XCLL\Directional Color\Green'));
    slCELL.Add('XCLL\Directional Color\Blue;' + GetElementEditValues(e, 'XCLL\Directional Color\Blue'));
    slCELL.Add('XCLL\Fog Color Near\Red;' + GetElementEditValues(e, 'XCLL\Fog Color\Red'));
    slCELL.Add('XCLL\Fog Color Near\Green;' + GetElementEditValues(e, 'XCLL\Fog Color\Green'));
    slCELL.Add('XCLL\Fog Color Near\Blue;' + GetElementEditValues(e, 'XCLL\Fog Color\Blue'));
    slCELL.Add('XCLL\Fog Color Far\Red;' + GetElementEditValues(e, 'XCLL\Fog Color\Red'));
    slCELL.Add('XCLL\Fog Color Far\Green;' + GetElementEditValues(e, 'XCLL\Fog Color\Green'));
    slCELL.Add('XCLL\Fog Color Far\Blue;' + GetElementEditValues(e, 'XCLL\Fog Color\Blue'));
    slCELL.Add('XCLL\Fog Near;' + GetElementEditValues(e, 'XCLL\Fog Near'));
    slCELL.Add('XCLL\Fog Far;' + GetElementEditValues(e, 'XCLL\Fog Far'));
    slCELL.Add('XCLL\Directional Rotation XY;' + GetElementEditValues(e, 'XCLL\Directional Rotation XY'));
    slCELL.Add('XCLL\Directional Rotation Z;' + GetElementEditValues(e, 'XCLL\Directional Rotation Z'));
    slCELL.Add('XCLL\Directional Fade;' + GetElementEditValues(e, 'XCLL\Directional Fade'));
    slCELL.Add('XCLL\Fog Clip Distance;' + GetElementEditValues(e, 'XCLL\Fog Clip Dist'));
    RemoveElement(e, 'XCLL');
  end;
  // remove music
  RemoveElement(e, 'XCMT');
  // remove water level
  RemoveElement(e, 'XCLW');
  // remove climate
  RemoveElement(e, 'XCCM');
  // remove water
  RemoveElement(e, 'XCWT');
end;  

//============================================================================
procedure BeforeREFR(e: IInterface);
var
  baseobj: IInterface;
  baseform: Cardinal;
  s: string;
  saved: boolean;
  i: integer;
begin
  baseobj := LinksTo(ElementBySignature(e, 'NAME'));
  if not Assigned(baseobj) then begin
    RemoveNode(e);
    Exit;
  end;
  
  // remove refs to objects that are not applicable to Skyrim
  s := Signature(baseobj);
  if (s = 'SBSP') or (s = 'SGST') then begin
    RemoveNode(e);
    Exit;
  end;
  
  saved := false;
  
  baseform := GetElementNativeValues(e, 'NAME');
  // convert refs from Oblivion.esm to Skyrim.esm
  if (baseform shr 24) = 0 then begin
    baseform := Obl2Sky(baseform);
    SetElementNativeValues(e, 'NAME', baseform);
  end;

  // store teleport data
  if ElementExists(e, 'XTEL') then begin
    baseform := GetElementNativeValues(e, 'XTEL\Door');
    if (baseform shr 24) > 0 then begin
      if not saved then begin
        slREFR.Add(IntToHex64(GetLoadOrderFormID(e), 8));
        saved := true;
      end;
      slREFR.Add('XTEL');
      slREFR.Add('XTEL\Door;' + IntToHex64(GetElementNativeValues(e, 'XTEL\Door'), 8));
      slREFR.Add('XTEL\Position/Rotation\Position\X;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Position\X'));
      slREFR.Add('XTEL\Position/Rotation\Position\Y;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Position\Y'));
      slREFR.Add('XTEL\Position/Rotation\Position\Z;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Position\Z'));
      slREFR.Add('XTEL\Position/Rotation\Rotation\X;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Rotation\X'));
      slREFR.Add('XTEL\Position/Rotation\Rotation\Y;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Rotation\Y'));
      slREFR.Add('XTEL\Position/Rotation\Rotation\Z;' + GetElementEditValues(e, 'XTEL\Position/Rotation\Rotation\Z'));
      RemoveElement(e, 'XTEL');
    end;
  end;

  // store lock data
  if ElementExists(e, 'XLOC') then begin
    i := GetElementNativeValues(e, 'XLOC\Lock Level');
    if i <> 255 then
    if i > 80 then i := 100 else
      if i > 60 then i := 75 else
        if i > 40 then i := 50 else
          if i > 20 then i := 25 else
            i := 1;
    baseform := GetElementNativeValues(e, 'XLOC\Key');
    if (baseform shr 24) = 0 then baseform := 0; // null for keys from oblivion.esm
    if not saved then begin
      slREFR.Add(IntToHex64(GetLoadOrderFormID(e), 8));
      saved := true;
    end;
    slREFR.Add('XLOC');
    slREFR.Add('XLOC\Level;' + IntToStr(i));
    slREFR.Add('XLOC\Key;' + IntToHex64(baseform, 8));
    slREFR.Add('XLOC\Flags;' + IntToStr(GetElementNativeValues(e, 'XLOC\Flags')));
    RemoveElement(e, 'XLOC');
  end;
  
  // convert map marker type
  if ElementExists(e, 'Map Marker') then begin
    i := GetElementNativeValues(e, 'Map Marker\TNAM\Type');
    if i = 1 then i := 5 else // camp
    if i = 2 then i := 4 else // cave
    if i = 3 then i := 1 else // city
    if i = 4 then i := 7 else // ayleid ruin
    if i = 5 then i := 6 else // fort ruin
    if i = 7 then i := 11 else // landmark
    if i = 8 then i := 14 else // tavern
    if i = 9 then i := 3 else // settlement
    if i = 10 then i := 34 else // daedric shrine
    if i = 11 then i := 34 else // oblivion gate
    if i = 12 then i := 11 else // door?
      i := 11;
    SetElementNativeValues(e, 'Map Marker\TNAM\Type', i);
  end;

  // remove enable refs linked to oblivion.esm refs
  if ElementExists(e, 'XESP') then begin
    baseform := GetElementNativeValues(e, 'XESP\Reference');
    if ((baseform shr 24) = 0) and (baseform <> $14) then
      RemoveElement(e, 'XESP');
  end;
  
  // not applicable to skyrim
  RemoveElement(e, 'XTRG');
  RemoveElement(e, 'XSED');
  RemoveElement(e, 'XLOD');
  RemoveElement(e, 'XHLT');
  RemoveElement(e, 'Unused');
  RemoveElement(e, 'XRTM');
  RemoveElement(e, 'XSOL');
  
end;  

//============================================================================
procedure BeforeACHR(e: IInterface);
var
  baseform: Cardinal;
  baseobj: IInterface;
begin
  baseobj := LinksTo(ElementBySignature(e, 'NAME'));
  if not Assigned(baseobj) then begin
    RemoveNode(e);
    Exit;
  end;
  
  baseform := GetElementNativeValues(e, 'NAME');
  // convert refs from Oblivion.esm to Skyrim.esm
  if (baseform shr 24) = 0 then begin
    baseform := $0100080C; //0Skeleton1Archer "Skeleton" [NPC_:0100080C]
    SetElementNativeValues(e, 'NAME', baseform);
  end;

  // remove enable refs linked to oblivion.esm refs
  if ElementExists(e, 'XESP') then begin
    baseform := GetElementNativeValues(e, 'XESP\Reference');
    if ((baseform shr 24) = 0) and (baseform <> $14) then
      RemoveElement(e, 'XESP');
  end;
  
  // not applicable to skyrim
  RemoveElement(e, 'Unused');
  RemoveElement(e, 'XLOD');
  RemoveElement(e, 'XMRC');
  RemoveElement(e, 'XHRS');
  RemoveElement(e, 'XRGD');
end;  

//============================================================================
procedure BeforePGRD(e: IInterface);
begin
  RemoveNode(e);
end;  

//============================================================================
procedure BeforeROAD(e: IInterface);
begin
  RemoveNode(e);
end;  

//============================================================================
function Initialize: integer;
begin
  slCELL := TStringList.Create;
  slREFR := TStringList.Create;
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;
  
  Obl2Sky(0);
end;

//============================================================================
function Process(e: IInterface): integer;
var
  s: signature;
  line: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;
  
  s := Signature(e);
  
  if (s = 'TES4') then BeforeTES4(e) else
  if (s = 'REGN') then BeforeREGN(e) else
  if (s = 'WRLD') then BeforeWRLD(e) else
  if (s = 'CELL') then BeforeCELL(e) else
  if (s = 'REFR') then BeforeREFR(e) else
  if (s = 'ACHR') then BeforeACHR(e) else
  if (s = 'ACRE') then BeforeACHR(e) else
  if (s = 'ROAD') then BeforeROAD(e) else
  if (s = 'PGRD') then BeforePGRD(e);

end;

//============================================================================
function Finalize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind CELL.txt';
  AddMessage('Saving ' + fname);
  slCELL.SaveToFile(fname);
  slCELL.Free;

  fname := ProgramPath + 'Edit Scripts\Skywind REFR.txt';
  AddMessage('Saving ' + fname);
  slREFR.SaveToFile(fname);
  slREFR.Free;

  sl.Free;
  slv.Free;

  EnableSkyrimSaveFormat;
end;

end.
