// transfer FLOR from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // FloraMushroom01 "Fly Amanita" [FLOR:0004D9FF]
  baserecord := RecordByFormID(FileByIndex(0), $0004D9FF, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find base record');
    Result := 1;
    Exit;
  end;
  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;
    slv.DelimitedText := sl[i];
    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    SetElementEditValues(r, 'FULL', slv[2]);
    SetElementEditValues(r, 'Model\MODL', slv[3]);
    if slv[4] <> '' then SetElementNativeValues(r, 'PFIG', StrToInt64('$' + slv[4])) else begin
      RemoveElement(r, 'PFIG');
      RemoveElement(r, 'RNAM');
      RemoveElement(r, 'SNAM');
      SetElementEditValues(r, 'PFPC\Spring', '0');
      SetElementEditValues(r, 'PFPC\Summer', '0');
      SetElementEditValues(r, 'PFPC\Fall', '0');
      SetElementEditValues(r, 'PFPC\Winter', '0');
    end;
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind FLOR.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  pfig: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'FLOR' then
    Exit;

  // export
  pfig := IntToHex(GetElementNativeValues(e, 'PFIG'), 8);
  if Copy(pfig, 1, 2) <> '01' then pfig := '';
  line := Format('%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    pfig
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
