// fixup for remaining issues

unit UserScript;

const
  plugin_name = 'skywind.esp';
  
var
  plugin: IInterface;

//============================================================================
function GetSOUNforSNDR(formid: Cardinal): Cardinal;
var
  i: integer;
  ents, ent: IInterface;
begin
  Result := 0;
  ents := GroupBySignature(plugin, 'SOUN');
  for i := 0 to ElementCount(ents) - 1 do begin
    ent := ElementByIndex(ents, i);
    if GetElementNativeValues(ent, 'SDSC') = formid then begin
      Result := GetLoadOrderFormID(ent);
      Exit;
    end;
  end;
end;

//============================================================================
function Initialize: integer;
var
  formid: Cardinal;
  i: integer;
  sndrs, ent, r, baserecord: IInterface;
begin
  // create SOUN records for SNDR
  // SNDR can't be placed directly by REFRs
  
  plugin := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(plugin), plugin_name) then begin
    AddMessage(plugin_name + ' must be loaded last');
    Result := 1;
    Exit;
  end;
  
  if Assigned(GroupBySignature(plugin, 'SOUN')) then
    Exit;

  sndrs := GroupBySignature(plugin, 'SNDR');
  if not Assigned(sndrs) then
    Exit;

  // AMBRumbleShakeGreybeards [SOUN:00000E06]
  baserecord := RecordByFormID(FileByIndex(0), $00000E06, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find SOUN base record');
    Result := 1;
    Exit;
  end;

  formid := $01300260; // free space after last SNDR record
  for i := 0 to ElementCount(sndrs) - 1 do begin
    ent := ElementByIndex(sndrs, i);
    r := wbCopyElementToFile(baserecord, plugin, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Result := 1;
      Exit;
    end;
    SetLoadOrderFormID(r, formid);
    SetElementEditValues(r, 'EDID', 'SOUN' + GetElementEditValues(ent, 'EDID'));
    SetElementNativeValues(r, 'SDSC', GetLoadOrderFormID(ent));
    Inc(formid);
  end;
  
end;


//============================================================================
function Process(e: IInterface): integer;
var
  s: string;
  ref: IInterface;
  entries, ent: IInterface;
  formid, flags: Cardinal;
  i: integer;
begin
  if wbGameMode <> gmTES5 then begin
    Result := 1;
    Exit;
  end;

  s := Signature(e);
  
  if s = 'ACHR' then begin
    formid := GetElementNativeValues(e, 'NAME');
    // replace oblivion refs and unknown objects with skeleton
    ref := LinksTo(ElementBySignature(e, 'NAME'));
    if (formid shr 24 = 0) or (Signature(ref) <> 'NPC_') then
      SetElementNativeValues(e, 'NAME', $0100080C); // 0Skeleton1Archer "Skeleton" [NPC_:0100080C]

    // unknown ownership from oblivion.esm
    if ElementExists(e, 'Ownership') then begin
      formid := GetElementNativeValues(e, 'Ownership\XOWN');
      if formid shr 24 = 0 then
        SetElementNativeValues(e, 'Ownership\XOWN', $01220001); // 0ThievesSGuild "Thieves Guild" [FACT:01220001]
    end;
  end;
  
  
  if s = 'REFR' then begin
    // refr can't place LVLN anymore in Skyrim
    ref := LinksTo(ElementBySignature(e, 'NAME'));
    if Signature(ref) = 'LVLN' then
      SetElementNativeValues(e, 'NAME', $0100080C); // 0Skeleton1Archer "Skeleton" [NPC_:0100080C]
    
    if Signature(ref) = 'SNDR' then begin
      formid := GetSOUNforSNDR(GetLoadOrderFormID(ref));
      if formid = 0 then begin
        AddMessage('No SOUN for SNDR ' + Name(ref));
        Result := 1;
        Exit;
      end;
      SetElementNativeValues(e, 'NAME', formid);
    end;
    
    // unknown ownership from oblivion.esm
    if ElementExists(e, 'Ownership') then begin
      formid := GetElementNativeValues(e, 'Ownership\XOWN');
      if formid shr 24 = 0 then
        SetElementNativeValues(e, 'Ownership\XOWN', $01220001); // 0ThievesSGuild "Thieves Guild" [FACT:01220001]
    end;

    // fix Mapmarker ref does not have map marker data
    if GetElementNativeValues(e, 'NAME') = $00000010 then begin
      if not ElementExists(e, 'Map Marker') then begin
        Add(e, 'Map Marker', True);
        SetElementNativeValues(e, 'Map Marker\TNAM', 11); // landmark
        SetElementEditValues(e, 'Map Marker\FULL', GetElementEditValues(e, 'EDID'));
      end else
        SetElementEditValues(e, 'Map Marker\FULL', Trim(GetElementEditValues(e, 'Map Marker\FULL')));
    end;
    
    // Skyrim doesn't use VWD groups?
    if GetIsVisibleWhenDistant(e) then SetIsVisibleWhenDistant(e, False);
    
    // Evidence container from skyrim.esm, won't work with SureAI's version
    if GetElementNativeValues(e, 'NAME') = $000267E7 then
      SetElementNativeValues(e, 'NAME', $011C007E)  // 0comUchestU02Umisc01 "Large Chest" [CONT:011C007E]

  end;

  
  if s = 'LAND' then begin
    entries := ElementByName(e, 'Layers');
    for i := 0 to ElementCount(entries) - 1 do begin
      ent := ElementByIndex(entries, i);
      //AddMessage(Path());
      if ElementExists(ent, 'BTXT') then begin
        formid := GetElementNativeValues(ent, 'BTXT\Texture');
        if formid shr 24 = 0 then
          SetElementNativeValues(ent, 'BTXT\Texture', $0102000A); // 0ACUdarkgravel [LTEX:0102000A]
      end;
      if ElementExists(ent, 'ATXT') then begin
        formid := GetElementNativeValues(ent, 'ATXT\Texture');
        if formid shr 24 = 0 then
          SetElementNativeValues(ent, 'ATXT\Texture', $01020025); // 0SandU02 [LTEX:01020025]
      end;
    end;
  end;


  if s = 'NPC_' then begin
    // unique actors can't have multiple references
    flags := GetElementNativeValues(e, 'ACBS\Flags');
    flags := flags and $FFFFFFDF;
    SetElementNativeValues(e, 'ACBS\Flags', flags);
    // remove factions except CreatureFaction "Creature Faction" [FACT:00000013]
    entries := ElementByName(e, 'Factions');
    for i := ElementCount(entries) - 1 downto 0 do begin
      ent := ElementByIndex(entries, i);
      if GetElementNativeValues(ent, 'Faction') <> $00000013 then
        RemoveElement(entries, i);
    end;
    // remove audio templates to use with SureAI version of skyrim.esm
    RemoveElement(e, 'CSCR');
  end;

  if s = 'FACT' then begin
    // set ownership for factions
    flags := GetElementNativeValues(e, 'DATA\Flags');
    flags := flags or $8000;
    SetElementNativeValues(e, 'DATA\Flags', flags);
  end;
  
  if s = 'CELL' then begin
    s := GetElementEditValues(e, 'FULL');
    if s <> '' then begin
      s := Trim(s);
      s := Copy(s, 1, 33); // 33 chars limit on cell names
      SetElementEditValues(e, 'FULL', s);
    end;
    // set water height for cells
    if GetElementNativeValues(e, 'DATA') and $02 > 0 then
      if GetElementEditValues(e, 'XCLW') = '-2147483648.000000' then
        SetElementEditValues(e, 'XCLW', 'Default') ;
  end;

  
end;

end.
