// transfer LTEX from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;
  TXST_FormID: Cardinal;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r, txst: IInterface;
  i: integer;
  typ, formid, matt: Cardinal;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  sl.LoadFromFile(fname);
  
  TXST_FormID := $01020090; // free space in LTEX

  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    
    typ := StrToInt(slv[3]);
    
    if typ = 0 then matt := $00012F34 else // STONE, MaterialStone [MATT:00012F34]
    if typ = 1 then matt := $00012F37 else // CLOTH, MaterialCloth [MATT:00012F37]
    if typ = 2 then matt := $00012F38 else // DIRT, MaterialDirt [MATT:00012F38]
    if typ = 3 then matt := $00012F39 else // GLASS, MaterialGlass [MATT:00012F39]
    if typ = 4 then matt := $00012F46 else // GRASS, MaterialGrass [MATT:00012F46]
    if typ = 5 then matt := $00012F3C else // METAL, MaterialSolidMetal [MATT:00012F3C]
    if typ = 6 then matt := $00012F3D else // ORGANIC, MaterialOrganic [MATT:00012F3D]
    if typ = 7 then matt := $00012F3F else // SKIN, MaterialSkin [MATT:00012F3F]
    if typ = 8 then matt := $00012F40 else // WATER, MaterialWater [MATT:00012F40]
    if typ = 9 then matt := $00012F41 else // WOOD, MaterialWoodLight [MATT:00012F41]
    if typ =10 then matt := $00012F36 else // HEAVY STONE, MaterialHeavyStone [MATT:00012F36]
    if typ =11 then matt := $00012F3B else // HEAVY METAL, MaterialHeavyMetal [MATT:00012F3B]
    if typ =12 then matt := $00012F42 else // HEAVY WOOD, MaterialWoodHeavy [MATT:00012F42]
    if typ =13 then matt := $00028E99 else // CHAIN, MaterialChain [MATT:00028E99]
    if typ =14 then matt := $00012F45 else // SNOW, MaterialSnow [MATT:00012F45]
      matt := $00012F34; // default stone
    
    // first need to create texture set
    // CityBannerMarkarthTower [TXST:0010EA44]
    formid := $0010EA44;
    txst := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(txst) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Exit;
    end;
    txst := wbCopyElementToFile(txst, skywind, True, True);
    if not Assigned(txst) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;
    SetLoadOrderFormID(txst, TXST_FormID);
    SetElementEditValues(txst, 'EDID', 'TXST_' + slv[1]);
    SetElementEditValues(txst, 'Textures (RGB/A)\TX00', 'landscape\' + slv[2]);
    SetElementEditValues(txst, 'Textures (RGB/A)\TX01', 'landscape\' + ChangeFileExt(slv[2], '') + '_n.dds');
    SetElementNativeValues(txst, 'DNAM', 1); // no specular map

    // LBlackreachDirt [LTEX:000D1BE4]
    formid := $000D1BE4;
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Exit;
    end;
   
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    SetElementNativeValues(r, 'TNAM', TXST_FormID);
    SetElementNativeValues(r, 'MNAM', matt);
    
    Inc(TXST_FormID);
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind LTEX.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'LTEX' then
    Exit;

  line := Format('%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'ICON'),
    IntToStr(GetElementNativeValues(e, 'HNAM\Material Type'))
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
