// transfer WEAP from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
  typ, formid: Cardinal;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    
    formid := $00012EB7; // default - iron sword
    typ := StrToInt(slv[6]);

    // Blade One Hand, IronSword "Iron Sword" [WEAP:00012EB7]
    if typ = 0 then formid := $00012EB7 else
    // Blade Two Hand, IronGreatsword "Iron Greatsword" [WEAP:0001359D]
    if typ = 1 then formid := $0001359D else
    // Blunt One Hand, IronMace "Iron Mace" [WEAP:00013982]
    if typ = 2 then formid := $00013982 else
    // Blunt Two Hand, IronWarhammer "Iron Warhammer" [WEAP:00013981]
    if typ = 3 then formid := $00013981 else
    // Staff, StaffFireball "Staff of Fireballs" [WEAP:00029B82]
    if typ = 4 then formid := $00029B82 else
    // Bow, HuntingBow "Hunting Bow" [WEAP:00013985]
    if typ = 5 then formid := $00013985;
    
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Result := 1;
      Exit;
    end;
   
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    if slv[2] <> '' then SetElementEditValues(r, 'FULL', slv[2]);
    SetElementEditValues(r, 'Model\MODL', slv[3]);
    // ENCH must be updated to be used for weapons
    //if slv[4] <> '' then SetElementNativeValues(r, 'EITM', StrToInt64('$' + slv[4])) else RemoveElement(r, 'EITM');
    //if slv[5] <> '' then SetElementEditValues(r, 'EAMT', slv[5]) else RemoveElement(r, 'EAMT');
    SetElementNativeValues(r, 'DNAM\Flags', StrToInt(slv[7]));
    SetElementEditValues(r, 'DNAM\Speed', slv[8]);
    // bad reach value for 0.00 that are in morroblivion
    //SetElementEditValues(r, 'DNAM\Reach', slv[9]);
    SetElementEditValues(r, 'DATA\Value', slv[10]);
    SetElementEditValues(r, 'DATA\Weight', slv[11]);
    SetElementEditValues(r, 'DATA\Damage', slv[12]);
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind WEAP.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  enam: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'WEAP' then
    Exit;

  enam := IntToHex(GetElementNativeValues(e, 'ENAM'), 8);
  if Copy(enam, 1, 2) <> '01' then enam := '';
  line := Format('%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    enam,
    GetElementEditValues(e, 'ANAM'),
    IntToStr(GetElementNativeValues(e, 'DATA\Type')),
    IntToStr(GetElementNativeValues(e, 'DATA\Flags')),
    GetElementEditValues(e, 'DATA\Speed'),
    GetElementEditValues(e, 'DATA\Reach'),
    GetElementEditValues(e, 'DATA\Value'),
    GetElementEditValues(e, 'DATA\Weight'),
    GetElementEditValues(e, 'DATA\Damage')
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
